require 'invaders_functions'
require 'object_loader'
require 'state_loader'
require 'font_loader'

function love.keypressed(key)
  if key == "escape" then
    love.event.push('quit')
  end

  if key == 'lctrl' then
    ctrl_timer:start()
  end

	if key == 'f' then
		FULLSCREEN = not FULLSCREEN
		love.window.setFullscreen( FULLSCREEN)
	end

  if key == 'up' then
		DRAW_DEPTH = DRAW_DEPTH + 1
    if DRAW_DEPTH > 3 then
			DRAW_DEPTH = 0
    end
	end

	if key == 'down' then
		DRAW_DEPTH = DRAW_DEPTH - 1
    if DRAW_DEPTH < 0 then
			DRAW_DEPTH = 0
    end
	end
	
	if key == " " and DISPLAY_RESTART then
	  DISPLAY_RESTART = false
	  speed_up()
	end
	
	if key == " " and DISPLAY_INIT then
	  hide_init()
	end

  invaders:keypressed(key)
end

local FADE_STARTED = false
function hide_init()
  if FADE_STARTED then return end
  local t = 2
  fade_timer:start()
  FADE_STARTED = true
  event:new(EVENT_MANAGER, {function() DISPLAY_INIT = false end, 1}, 2)
end

function slow_down()
  SLOW_DOWN = true
  ctrl_timer:start()
end

function speed_up()
  SLOW_DOWN = false
end

DISPLAY_INIT = true
DISPLAY_RESTART = false
DR_X = nil
DR_Y = nil
function display_restart(x, y)
  DISPLAY_RESTART = true
  DR_X = x
  DR_Y = y
end

function love.keyreleased(key)
  invaders:keyreleased(key)
end
function love.mousepressed(x, y, button)
  invaders:mousepressed(x, y, button)
end
function love.mousereleased(x, y, button)
  invaders:mousereleased(x, y, button)
end

function love.load(arg)
  -- GLOBALS -------------------------------------------------------------------
  lg = love.graphics
  lw = love.window
  lf = love.filesystem
  lk = love.keyboard
  ARGS = arg
  DEBUG = true
  
  function lg.resize_window(width, height)
    SCR_WIDTH, SCR_HEIGHT = width, height
    
    -- mouse_input scaling depends on window size
    MOUSE_INPUT:init()
    lw.setMode(width, height, {fullscreen = FULLSCREEN})
  end
  
  -- globals
  SCR_WIDTH  = arg[2] or 800
  SCR_HEIGHT = arg[3] or 600
  FULLSCREEN = false
  RANDCOLOUR = arg[5] and true
  MOUSE_INPUT = nil
  TILE_WIDTH = 40
  TILE_HEIGHT = 40
  CELL_WIDTH = 128                        -- collider cell width
  CELL_HEIGHT = 128
  MAX_IMAGE_WIDTH = 2048                  -- in pixels
  MAX_IMAGE_HEIGHT = 2048
  DRAW_DEPTH = 0
  SLOW_DOWN = false
  love.mouse.setVisible(false)

  -- colors
  C_WHITE = {255, 255, 255, 255}
  C_BLACK = {0, 0, 0, 255}
  C_RED = {255, 0, 0, 255}
  C_BORDER = {0, 0, 0, 80}
  CL1_RED = {240, 77, 99}
  CL1_TEAL = {88, 222, 246}
  CL1_RED_D = {171, 9, 15, 255}
  CL1_TEAL_D = {114, 178, 173, 255}
  
  -- channels
  RED, GREEN, BLUE, ALPHA = 1, 2, 3, 4
  
  ------------------------------------------------------------------------------

  -- find user's resolution
  --love.graphics.setMode(0, 0)
  --local max_width = love.graphics.getWidth()
  --local max_height = love.graphics.getHeight()
  --love.graphics.setMode(SCR_WIDTH, SCR_HEIGHT, false, true)
  local resolutions = get_supported_resolutions(1920, 1080)
  local max_width = resolutions[1].width
  local max_height = resolutions[1].height
  
  -- assets
  load_fonts()
  load_objects()
  load_states()
  
  -- keeps track of time for the game
  master_timer = master_timer:new()
  
  -- input
  MOUSE_INPUT = mouse_input:new()
  
  -- states
  invaders = state_manager:new()
  --invaders:add_state(first_state)
  invaders:add_state(level1_load_state)
  invaders:add_state(level1_game_state)
  
  lg.resize_window(max_width, max_height)
  invaders:load_state(level1_load_state)


  
  ctrl_master = master_timer:new()
  ctrl_timer = timer:new(ctrl_master, 0.4)
  fade_timer = timer:new(ctrl_master, 2)
  
  EVENT_MANAGER = event_manager:new()
end

function love.update(dt)
  EVENT_MANAGER:update(dt)

  if love.keyboard.isDown('lctrl') or SLOW_DOWN then 
    local min = 1
    local max = 10
    local current = min + ctrl_timer:progress() * (max - min)
    dt = dt / current
  end


  master_timer:update(dt)
  ctrl_master:update(dt)
  if master_timer.is_stopped then dt = 0 end
  MOUSE_INPUT:update(dt)
  invaders:update(dt)
end

function love.draw()
  invaders:draw()
  MOUSE_INPUT:draw()
  lg.setFont(courier_small)
  lg.setColor(255, 0, 0, 90)
  lg.print("FPS "..love.timer.getFPS(), 0, 0)
  
  if DISPLAY_RESTART then
    lg.setColor(230, 245, 255, 200)
    lg.print("You crashed and exploded. Press [spacebar] to live again.\n\n"..
             "Press [Esc] to exit.", DR_X, DR_Y)
  end
  
  if DISPLAY_INIT then
    local min, max = 0, 200
    local alpha = max
    
    if FADE_STARTED then
      alpha = min + (1-fade_timer:progress()) * (max - min)
    end
    
    lg.setColor(230, 245, 255, alpha)
    lg.print("Click and drag your mouse to move\nTap the [spacebar] to use your sonar", 100, 200)
  end
  
end














