
--##########################################################################--
--[[----------------------------------------------------------------------]]--
-- bullet object
--[[----------------------------------------------------------------------]]--
--##########################################################################--
local bullet = {}
bullet.table = BULLET
bullet.parent = nil         -- gun that sent the bullet
bullet.owner = nil          -- object that triggered the gun
bullet.isdead = false

-- position
bullet.mpoint = nil
bullet.pos = nil
bullet.dir = nil

-- movement
bullet.collider = nil
bullet.fire_speed = 300
bullet.vel = nil
bullet.depth = 0
bullet.max_depth = 1

local bullet_mt = { __index = bullet }
function bullet:new(level, position, direction, velocity)

	local vel = bullet.fire_speed * direction + velocity
	local mpoint = map_point:new(level, position)
	
	local collider = level:get_collider()
	local camera = level:get_camera()
	
  return setmetatable({ pos = position,
                        dir = direction,
                        vel = vel,
                        mpoint = mpoint,
                        collider = collider,
                        camera = camera}, bullet_mt)
end

function bullet:init(position, direction, velocity)
	self.vel = bullet.fire_speed * direction + velocity
	self.depth = bullet.depth
	self:set_position(position)
	self:set_direction(direction)
	self.collider:add_object(self.mpoint, self)
	self.isdead = false
	
	if not self.mpoint.tile.walkable then
		self:die()
	end
end

function bullet:set_parent(obj)
	self.parent = obj
end
function bullet:set_owner(obj)
	self.owner = obj
end
function bullet:set_position(pos)
	self.pos = pos
	self.mpoint:update_position(pos)
end
function bullet:set_direction(dir)
	self.dir = dir
end

function bullet:die()
	self.isdead = true
	self.collider:remove_object(self.mpoint)
end

------------------------------------------------------------------------------
function bullet:update(dt)
	-- move point
	local newpos = self.pos + self.vel * dt
	local point = self.mpoint	
	point:set_position(newpos)
	point:update(dt)
	local c, n, p, off, tile = point:get_collision_data()
	
	-- collision with tile
	if c then
		self.vel = get_bounce_velocity(self.vel, n, 0.8, 0.9)
		self.depth = self.depth + 1
		self:set_direction(self.vel:normalize())
		point:update_position(p)
		newpos = p
		if self.depth > self.max_depth then
			self:die()
		end
	end
	
	self.pos = newpos
	self.collider:update_object(point)
	
	-- check if in view
	local pad = 1000
	local x, y, w, h  = self.camera:get_viewport()
	local x, y = x - pad, y - pad
	local w, h = w + 2 * pad, h + 2 * pad
	local nx, ny = newpos.x, newpos.y
	if not (nx > x and nx < x + w and ny > y and ny < y + h) then
	  self:die()
	end
	
end

------------------------------------------------------------------------------
function bullet:draw()
end

return bullet




























