
--##########################################################################--
--[[----------------------------------------------------------------------]]--
-- gun object
--[[----------------------------------------------------------------------]]--
--##########################################################################--
local gun = {}
gun.table = GUN
gun.debug = false
gun.parent = nil
gun.pos = nil
gun.dir = nil

gun.num_bullets = 1000
gun.bullets = nil					-- active bullets
gun.deadpool = nil        -- pool of ready to use bullets

local gun_mt = { __index = gun }
function gun:new(level, position, direction)
	-- create bullets with dummy values
	local ammo = {}
	for i=1,gun.num_bullets do
		ammo[i] = bullet:new(level, position, NORMAL[UP], VECT_ZERO)
	end
	
	local collider = level:get_collider()
	
  return setmetatable({ pos = position,
                        dir = direction,
                        deadpool = ammo,
                        bullets = {},
                        collider = collider}, gun_mt)
end

-- pos = vector
function gun:set_position(pos)
	self.pos = pos
end
function gun:get_position()
	return self.pos
end

-- dir = normalized vector
function gun:set_direction(dir)
	self.dir = dir
end
function gun:get_direction()
	return self.dir
end

function gun:set_parent(obj)
	self.parent = obj
end

local gradient = require("flash")
function gun:shoot()
	if #self.deadpool == 0 then return end
	
	local bullet = self.deadpool[#self.deadpool]
	self.deadpool[#self.deadpool] = nil
	self.bullets[#self.bullets + 1] = bullet
	
	bullet:set_parent(self)
	bullet:set_owner(self.parent)
	
	bullet:init(self.pos, self.dir, self.parent:get_velocity() * DSCALE)
	
	bullet.color = gradient[math.random(30,#gradient)]
end

------------------------------------------------------------------------------
function gun:update(dt)
	local bullets = self.bullets
	for i=#bullets,1,-1 do
		local bullet = bullets[i]
		bullet:update(dt)
		if bullet.isdead then
			table.remove(bullets, i)
			self.deadpool[#self.deadpool+1] = bullet
			
			--self.collider:remove_object(bullet.mpoint)
		end
	end
end

------------------------------------------------------------------------------
function gun:draw()
	local bullets = self.bullets
	lg.setPointSize(5)
	for i=1,#bullets do
	  --[[
		local pos = bullets[i].pos
		local dir = bullets[i].dir
		local perp = vector2:new(dir.y, -dir.x)
		local len = 10
		local off = 10
		local end1 = pos - dir * len + perp * off
		local end2 = pos - dir * len - perp * off
		
		lg.setColor(0,0,0,255)
		if math.random() > 0.50 then
			if math.random() > 0.50 then
				lg.setColor(0, 0, math.random() * 200)
		  else
		  	lg.setColor(math.random() * 100, 0, 0)
		  end
		end
		
		lg.setLineWidth(3)
		lg.line(pos.x, pos.y, end1.x,  end1.y)
		lg.line(pos.x, pos.y, end2.x,  end2.y)
		]]--
		
		--bullets[i].mpoint:draw()
		local x, y = bullets[i].mpoint.pos.x, bullets[i].mpoint.pos.y
		lg.setColor(bullets[i].color)
		lg.point(x, y)
		
		--[[
		lg.setColor(math.random(255), 0, 0, 255)
		lg.setPoint(10, "smooth")
		lg.point(pos.x, pos.y)
		]]--
	end
end

------------------------------------------------------------------------------
function gun:draw_debug()
	local pos = self.pos
	local dir = self.dir
	local length = 10
	local endpos = pos + length * dir
	lg.setColor(0,0,0,255)
	lg.setLineWidth(3)
	lg.line(pos.x, pos.y, endpos.x, endpos.y)
	
	print("asd")
end

return gun
































