time = 0
--##########################################################################--
--[[----------------------------------------------------------------------]]--
-- hero object
--[[----------------------------------------------------------------------]]--
--##########################################################################--
local hero = {}
hero.table = HERO
hero.debug = false
hero.map = nil

-- input
hero.mouse = nil
hero.click_timer = nil              
hero.dclick_time = 0.200              -- in milliseconds
hero.dclick_effect_time = 0.500       -- in milliseconds
hero.dclick_timer = nil             -- timer for double click effect
hero.dclick_dir = nil               -- direction when mouse was double clicked
hero.input_radius = 150             -- pixels, mouse boundary radius
hero.input_direction = nil
hero.input_power = 0
hero.thrust_time = 0.250              -- time for thrust to reach full power
hero.input_thrust = 0               -- [0,1], thrust value

-- direction
hero.direction = vector2:new(0, -1) -- current direction
hero.num_dirs = 30                  -- number of past direcitons to remember
hero.dir_history = nil							-- array of past num_dirs directions

-- movement
hero.point = nil
hero.pos = VECT_ZERO
hero.max_force = 12
hero.max_velocity = 10
hero.min_velocity = 3
hero.velocity_limit = hero.max_velocity
hero.friction = 1.5

-- camera
hero.camera_point = nil
hero.camera_target = VECT_ZERO
hero.camera = nil
hero.cam_radius = 0          -- current dist camera target is away from hero
hero.act_cam_radius = 0      -- actual radius (target radius for cam_radius)
hero.cam_max_radius = 180
hero.shrink_speed = 5
hero.enlarge_speed = 40
hero.min_approach = 0.4      -- approaching slowing factor for camera arriving
hero.max_approach = 1.5      -- at the camera target

-- collision
hero.bbox = nil
hero.bbox_width = 15
hero.bbox_height = 15
hero.body = nil
hero.m_point = nil

-- weapon
hero.shoot_button = 'lshift'
hero.gun = nil
hero.shoot_time = 0.001
hero.shoot_timer = nil        -- keeps track of time since last shoot

hero.exploded = false
hero.refresh_timer = nil
hero.sonar_points = {}

local hero_mt = { __index = hero }
function hero:new(level, pos)
  local hero = setmetatable({}, hero_mt)
  local master_timer = level:get_master_timer()

	local point = physics.point:new(pos)
	
	-- input timers
	local click_timer = timer:new()
	local dclick_timer = timer:new(master_timer, hero.dclick_effect_time)
	
	-- direction
	local dir_history = {}
	for i=1,hero.num_dirs do
		dir_history[i] = hero.direction
	end
	
	--camera
	local target = vector2:new(SCR_WIDTH/2, SCR_HEIGHT/2)
	local cam_point = physics.steer:new()
	cam_point:set_target(target)
	cam_point:set_position(target)
	cam_point:set_mass(0.25)
	cam_point:set_force(5)
	cam_point:set_max_speed(20)
  cam_point:set_radius(300)
	
  -- bounding box
  local bx = pos.x - 0.5 * hero.bbox_width
  local by = pos.y - 0.5 * hero.bbox_height
  local bbox = bbox:new(bx, by, hero.bbox_width, hero.bbox_height)
  bbox.parent = hero
  
  local collider = level:get_collider()
  collider:add_object(bbox, hero)
  
  -- weapon
  hero_gun = gun:new(level, pos, hero.direction)
  hero_gun:set_parent(hero)
  local shoot_timer = timer:new(master_timer, hero.shoot_time)
  shoot_timer:start()
  
  -- body
  local points = {vector2:new(bbox.x, bbox.y),
									vector2:new(bbox.x + bbox.width, bbox.y),
									vector2:new(bbox.x + bbox.width, bbox.y + bbox.height),
									vector2:new(bbox.x, bbox.y + bbox.height)}
  local body = map_body:new(level, points)
  body:set_origin(pos)
  body:set_parent(hero)
  
  local m_point = map_point:new(level, pos)
  
  --[[
  -- ray test
  local rays = {}
  local num_rays = 300
  for i=1,num_rays do
    local pos = pos
    local vi = vector2:new(0, 0)
    local dir = hero.direction
    local angle = math.atan2(dir.x, dir.y)
    local angle_width = 0.4
    local min = angle - angle_width
    local max = angle + angle_width
    local speed = 0.1  + 0.00 * math.random()
    speed = 0.05
    local phase = math.random() * 2* math.pi
    
    local angle = min + (0.5 * math.sin(time * speed + phase) + 0.5) * (max - min)
    local dir = vector2:new(math.sin(angle), math.cos(angle))
    local ray = ray:new(level, pos, dir)
    ray.speed = speed
    ray.phase = phase
    ray.div = angle_width
    rays[#rays + 1] = ray
  end
  ]]--
  
  -- sonar gun
  local sgun = sonar_gun:new(level, pos, hero.direction)
  
  hero.level = level
  hero.point = point
  hero.click_timer = click_timer
  hero.dclick_timer = dclick_timer
  hero.dir_history = dir_history
  hero.camera_point = cam_point
  hero.bbox = bbox
  hero.map = map
  hero.camera = level:get_camera()
  hero.m_point = m_point
  hero.shoot_timer = shoot_timer
  hero.gun = hero_gun
  hero.sgun = sgun
  hero.body = body
  hero.collider = collider
  hero.master_timer = master_timer
  hero.mouse = level:get_mouse()
  hero.rays = rays
  hero.refresh_timer = timer:new(master_timer, 0.75)
  
  return hero
end

function hero:shoot()
	self.gun:shoot()
end

function hero:shoot_sonar()
  self.sgun:shoot()
  
  self.shoot_pos = vector2:new(self.pos.x, self.pos.y)
  self.shoot_pos.dir = vector2:new(self.direction.x, self.direction.y)
  self.sonar_points[#self.sonar_points + 1] = self.shoot_pos
end

------------------------------------------------------------------------------
function hero:update(dt)
  time = time + dt

	self:_update_input(dt)
	self:_update_direction(dt)
	self:_update_position(dt)
	self:_update_camera_target(dt)
	self:_update_collision(dt);
	self:_update_weapon(dt)
	
  if lk.isDown('k') then
    return
  end
  
  --[[
  local rays = self.rays
  for _,ray in ipairs(rays)
  do
    local r = 0.80
    if lk.isDown('lctrl') then
      r = 0.96
    end
    if math.random() > r then
      local angle = math.atan2(self.direction.x, self.direction.y)
      local min = angle - ray.div
      local max = angle + ray.div
      local angle = min + (0.5 * math.sin(time * ray.speed + ray.phase) + 0.5) * (max - min)
      local dir = vector2:new(math.sin(angle), math.cos(angle))
      ray:set(self.pos, dir)
      ray:cast(3)
    end
  end
  ]]--
  
  if self.exploded and self.refresh_timer:progress() == 1 then
    self.exploded = false
  end
  
end

------------------------------------------------------------------------------
function hero:_update_weapon(dt)
	-- set orientation
	local dir = self.direction
	local pos = self.pos + 10 * dir
	self.gun:set_position(pos)
	self.gun:set_direction(dir)
	self.gun:update(dt)
	
	-- player shooting
	if love.keyboard.isDown(self.shoot_button) then
		if not self.shoot_timer:isrunning() then
			self:shoot()
			self.shoot_timer:start()
		end
	end
	
	
	self.sgun:set_position(self.pos)
	self.sgun:set_direction(dir)
	self.sgun:update(dt)
	
end

function hero:find_new_home()
  local level_map = self.level:get_level_map()
  
  local tmaps = level_map:_get_loaded_tile_maps()
  for i=1,30 do
    local tmap = tmaps[math.random(1,#tmaps)]
    local bbox = tmap.bbox
    local pad = 1000
    local x, y, w, h = bbox.x + pad, bbox.y + pad, bbox.width - 2 * pad, bbox.height - 2 * pad
    
    for j = 1,20 do
      local rx, ry = math.random(x, x + w), math.random(y, y + h)
      local pos = vector2:new(rx, ry)
      local tile = tmap:get_tile_at_position(pos)
      if tile.walkable and not tile.diagonal then
        self:set_position(pos)
        self:update(1/60)
        self.level:get_camera():set_target(pos, true)
        print("transport to: ", rx, ry)
        return
      end
    end
    
  end
  
end

local new_home = true
function hero:explode()
  if self.exploded then
    return
  end
  
  self:find_new_home()
  if not new_home then
    new_home = true
    return
  end
  
  self.sonar_points = {}
  self.sonar_point = nil
  
  event:new(EVENT_MANAGER, {function() slow_down(); self:shoot_sonar() end, 1}, 0.5)
  event:new(EVENT_MANAGER, {function() display_restart(100, 100) end, 1}, 0.7)
  
  local num = 200
  local v = vector2:new(0, 0)
  local pos = self.pos
  self.gun:set_position(pos)
  for i=1,num do
    local dir = vector2:new(math.random(-1000,1000),math.random(-1000,1000))
    dir = dir:normalize() * math.random()
	  self.gun:set_direction(dir)
	  self.gun:shoot()
  end
  
  self.refresh_timer:start()
  self.exploded = true
  
  --slow_down()
end

------------------------------------------------------------------------------
function hero:_update_collision(dt)
	-- update position of bbox
	local bbox = self.bbox
	local pos = self:get_position()
  bbox.x =  pos.x - 0.5 * hero.bbox_width
  bbox.y = pos.y - 0.5 * hero.bbox_height
  self.collider:update_object(bbox)
  
  -- update position of map point
  self.m_point:set_position(pos)
  self.m_point:update(dt)
  local c, n, p, tile = self.m_point:get_collision_data()
  if c then
  	self.point.vel = get_bounce_velocity(self.point.vel, n, 0.9, 0.3)
  	self:set_position(p)
  end
  
  -- update position of body
  self.body:set_position(self:get_position())
  self.body:update(dt)
  local c, n, p, offset, tiles = self.body:get_collision_data()
  if c then
  	local new_pos = self.body:get_position() + offset
  	local success, safe_pos = self.body:update_position(new_pos)
  	if not success then
  		new_pos = safe_pos
  	end
  	
  	self:set_position(new_pos)
  	self.point.vel = get_bounce_velocity(self.point.vel, n, 0.95, 0.3)
  	
  	self:explode()
  end
  
end

------------------------------------------------------------------------------
function hero:_update_position(dt)
	-- calculate pushing force, limit max_velocity
	self.limit_velocity = self.max_velocity
	if self.input_direction then
		local thrust = self.input_thrust
		local power = self.input_power
		local max_force = self.max_force
		local dir = self.input_direction

		local force = thrust * power * max_force * dir
		self.point:add_force(force)
	end

	-- set velocity limit
	local vmag = self.point:get_velocity():mag()
	if self.input_power == 0 then
		self.limit_velocity = self.max_velocity
	else
		local min, max = self.min_velocity, self.max_velocity
		local limit = math.max(min + self.input_power * (max - min), vmag)
		limit = math.min(limit, self.max_velocity)
		self.limit_velocity = limit
	end
	
	-- limit velocity
	local vdir = self.point:get_velocity():normalize()
	if vmag > self.limit_velocity then
		self.point:set_velocity(self.limit_velocity * vdir)
	end
	
	-- calculate friction force (only when idle)
	if self.input_power == 0 then
		local vmag = self.point:get_velocity():mag()
		local force_fr = -self.friction * vdir *  vmag
		self.point:add_force(force_fr)
	else
		local vmag = self.point:get_velocity():mag()
		local force_fr = -self.friction * vdir *  vmag / 4
		self.point:add_force(force_fr)
	end
	
	self.point:update(dt)
	self.pos = self.point:get_position()
	
end

------------------------------------------------------------------------------
function hero:_update_direction(dt)
	local next_dir = self.input_direction or self.direction
	local past_dirs = self.dir_history
	
	-- remove value at head, and insert new at tail
	table.remove(past_dirs, 1)
	past_dirs[self.num_dirs] = next_dir
	
	-- find average direction
	local avg = vector2:new(0, 0)
	for i=1,#past_dirs do
		local dir = past_dirs[i]
		avg = avg + dir
	end
	self.direction = avg:normalize()
end

------------------------------------------------------------------------------
function hero:_update_input()
  local mouse = self.mouse
  
	-- check whether mouse has been clicked/double clicked
	local double_clicked = false
	local clicked = mouse:was_left_pressed()
	if clicked then
		local time_passed = self.click_timer:time_elapsed()
	
		self.click_timer:reset()
		self.click_timer:start()
		self.dclick_timer:reset()    -- cancels double click effect
		
		if time_passed and time_passed < self.dclick_time then
			double_clicked = true
			self.click_timer:reset()
		end
	end
	
	-- start double clicked effect timer and set direction
	if double_clicked then
		self.dclick_timer:start()
		local dclick_dir = mouse:get_position() - self:get_screen_position()
		self.dclick_dir = dclick_dir:normalize()
	end
	
	-- make sure the mouse position doesnt move with the screen
	local mouse_pos = mouse:get_position()
	local screen_pos = self:get_screen_position()
	if self.input_length and mouse:is_down('l') then
		local mouse_diff = mouse_pos - self.last_mouse_pos
		local new_pos = screen_pos + self.input_direction * self.input_length + mouse_diff
		mouse:set_position(new_pos)
		mouse_pos = new_pos
	end
	self.last_mouse_pos = mouse_pos
	
	-- calc direction/power of input
	local direction, length
	local power = 0
	if mouse:is_down('l') then		
		direction = mouse_pos - screen_pos
		length = direction:mag()
		direction = direction:normalize()	
		
		-- limit mouse position
		if length > self.input_radius then
			local new_pos = screen_pos + self.input_radius * direction
			length = self.input_radius
		end
		
		power = length / self.input_radius
	end
	
	if not direction and self.dclick_timer:isrunning() then
		direction = self.dclick_dir
	end
	
	-- calculate thrust
	local time = mouse:get_left_time_pressed()
	local thrust = 1
	if time < self.thrust_time then
		thrust = time / self.thrust_time
	end
	
	self.input_direction = direction
	self.input_power = power
	self.input_length = length
	self.input_thrust = thrust
	
	-- keyboard input test
	if love.keyboard.isDown('w', 'a', 's', 'd') then
	  local lk = love.keyboard
	  local dir
	  if     lk.isDown('a') and lk.isDown('w') then
	    dir = NORMAL[UPLEFT]
	  elseif lk.isDown('w') and lk.isDown('d') then
	    dir = NORMAL[UPRIGHT]
	  elseif lk.isDown('s') and lk.isDown('d') then
	    dir = NORMAL[DOWNRIGHT]
	  elseif lk.isDown('a') and lk.isDown('s') then
	    dir = NORMAL[DOWNLEFT]
	  elseif lk.isDown('w') then
	    dir = NORMAL[UP]
	  elseif lk.isDown('a') then
	    dir = NORMAL[LEFT]
	  elseif lk.isDown('s') then
	    dir = NORMAL[DOWN]
	  elseif lk.isDown('d') then
	    dir = NORMAL[RIGHT]
	  end
	
		self.input_direction = dir
		self.input_power = 1
		self.input_length = length
		self.input_thrust = 1
	end
	
end

------------------------------------------------------------------------------
function hero:_update_camera_target(dt)
	-- calc camera radius
	local vmag = self.point:get_velocity():mag()
	local vratio = vmag / self.max_velocity
	local target_radius = vratio * self.cam_max_radius
	
	-- set radius based on target radius
	local radius = self.cam_radius
	if radius < target_radius then
		radius = math.min(radius + self.enlarge_speed * dt, target_radius)
	else
		radius = math.max(radius - self.shrink_speed * dt, target_radius)
	end
	self.cam_radius = radius

	-- update approaching behavior of camera depending on player speed
	local cam_target = self.camera.target
	local vratio = self.point.vel:mag() / self.max_velocity
	local minsf, maxsf = self.min_approach, self.max_approach
	cam_target:set_approach_factor(minsf + vratio * (maxsf - minsf))
	
	-- place camera target on the camera radius in direction of player
	self.camera_target = self.point.pos + radius * self.direction
end


------------------------------------------------------------------------------
function hero:get_camera_target()
	return self.camera_target
end

function hero:get_screen_position()
	return (self.pos - self.camera:get_pos()) * self.camera.scale
end

function hero:get_position()
	return self.pos
end

function hero:set_position(pos)
	self.pos = pos
	self.point:set_position(pos)
	self.m_point:update_position(pos)
	
	local bbox = self.bbox
	local pos = self:get_position()
  bbox.x =  pos.x - 0.5 * hero.bbox_width
  bbox.y = pos.y - 0.5 * hero.bbox_height
  self.collider:update_object(bbox)
end

function hero:set_camera(cam)
	self.camera = camera
end

function hero:get_velocity()
	return self.point:get_velocity()
end

function hero:draw()

	lg.setColor(100, 100, 100, 255)
	lg.setLineWidth(2)
	lg.circle('line', self.pos.x, self.pos.y, 5)
	
	local p1 = self.pos
	local p2 = p1 + 15 * self.direction
	lg.line(p1.x, p1.y, p2.x, p2.y)
	
	self.gun:draw()
	self.sgun:draw()
	
	self.m_point:draw()
	
	--[[
  if not lk.isDown('k') then
    for _,ray in ipairs(self.rays) do
      ray:draw()
    end
  end
  ]]--
  
  if self.shoot_pos then
    local alpha = 60
    local dec = 10
    lg.setColor(240,240,255,70)
    
    --lg.point(self.shoot_pos:get_vals())

    for i=1,#self.sonar_points do
      lg.setColor(255, 30, 0, alpha)
      local p1 = self.sonar_points[i]
      local dir = p1.dir
      local len = 15
      local p2 = p1 + len * dir
      lg.point(p1:get_vals())
      lg.line(p1.x, p1.y, p2.x, p2.y)
    end
  end
	
	if self.debug then
		lg.setColor(0, 255, 0, 255)
		lg.setLineWidth(1)
		lg.circle('line', self.pos.x, self.pos.y, self.cam_max_radius)
		lg.setColor(0, 255, 255, 255)
		lg.circle('line', self.pos.x, self.pos.y, self.cam_radius)
		
		local w = 10
		local pos = self.camera_target
		lg.setColor(0, 255, 0, 255)
		lg.rectangle('line', pos.x - w/2, pos.y - w/2, w, w)
		
		-- bbox
		lg.setColor(0, 200, 0, 255)
		self.bbox:draw()
		
		--weapon
		self.gun:draw_debug()
		
		-- body
		self.body:draw()
	end
end

return hero



