-- direction enums
UP        = 1
UPRIGHT   = 2
RIGHT     = 3
DOWNRIGHT = 4
DOWN      = 5
DOWNLEFT  = 6
LEFT      = 7
UPLEFT    = 8

-- normals
local diag = 1/math.sqrt(2)
NORMAL = {}
NORMAL[UP]        = vector2:new(0, -1)
NORMAL[UPRIGHT]   = vector2:new(diag, -diag)
NORMAL[RIGHT]     = vector2:new(1, 0)
NORMAL[DOWNRIGHT] = vector2:new(diag, diag)
NORMAL[DOWN]      = vector2:new(0, 1)
NORMAL[DOWNLEFT]  = vector2:new(-diag, diag)
NORMAL[LEFT]      = vector2:new(-1, 0)
NORMAL[UPLEFT]    = vector2:new(-diag, -diag)

--##########################################################################--
--[[----------------------------------------------------------------------]]--
-- tile object
--[[----------------------------------------------------------------------]]--
--##########################################################################--
local tile = {}
tile.table = TILE
tile.parent = nil    -- tile is child of a tile_map
tile.chunk = nil     -- chunk that the tile belongs to
tile.chunk_id = nil  -- id value that chunk uses to reference tile
tile.x = nil         -- world coordinate
tile.y = nil
tile.batch_x = nil   -- chunk sprite batch coordinate
tile.batch_y = nil
tile.neighbours = nil
tile.is_updating = false

-- attributes
tile.type = nil
tile.gradient = nil
tile.quad = nil
tile.walkable = true

-- diagonal data
tile.is_diagonal = false
tile.parent_tile = nil
tile.rotation = nil          -- in radians (multiples of pi / 2)
tile.direction = nil         -- NORMAL index as (2, 4, 6 or 8)
tile.diagonal = nil          -- reference to diagonal tile

local tile_mt = { __index = tile }
function tile:new()
  return setmetatable({}, tile_mt)
end

-- tile type is an integer, types found in tile_types.lua
function tile:set_type(tile_type)
  local old_type = self.type
  self.type = tile_type
  
  if     tile_type == T_WALK then
    self.walkable = true
  elseif tile_type == T_WALL then
    self.walkable = false
  else
    print("ERROR in tile:set_type() - type not recognized")
    return
  end
  
  if self.chunk and old_type then
    self.chunk:update_type_list(self, old_type, tile_type)
  end
end

function tile:set_diagonal_tile(direction, tile_type, quad, gradient)
	local diag_tile = tile:new(true)
	diag_tile.x = self.x
	diag_tile.y = self.y
	diag_tile.is_diagonal = true
	diag_tile.parent_tile = self
	diag_tile.direction = direction
	diag_tile.rotation = (0.5 * direction - 1) * 0.5 * math.pi
	
	diag_tile:set_quad(quad)
	diag_tile:set_gradient(gradient)
	diag_tile:set_type(tile_type)
	
	self.diagonal = diag_tile
end

function tile:has_diagonal_tile()
	if self.diagonal then
		return true
	end
	return false
end

function tile:is_diagonal_tile()
	return self.is_diagonal
end

function tile:set_gradient(gradient)
 self.gradient = gradient
end

function tile:get_type()
  return self.type
end

-- tiles = {up, upright, right, downright, down, downleft, left, upright}
function tile:set_neighbours(tiles)
  self.neighbours = tiles
end

-- parent is a tile_map object
function tile:set_parent(parent) self.parent = parent end
function tile:set_chunk(chunk) self.chunk = chunk end

function tile:set_quad(quad)
	self.quad = quad
	if self.chunk then
  	self.chunk:update_quad(self)
  end
end

function tile:set_chunk_id(id)
	self.chunk_id = id
end

-- position of tile in relation to top left corner of spritebatch
function tile:set_spritebatch_position(x, y)
  self.batch_x = x
  self.batch_y = y
end

function tile:halt_update()
	self.is_updating = false
end

function tile:start_update()
	self.is_updating = true
	self.chunk:add_tile_to_update_list(self)
end


------------------------------------------------------------------------------
function tile:update(dt)

end

------------------------------------------------------------------------------
function tile:draw()
end

return tile











