local level1_load_state = state:new()
level1_load_state.label = 'string'
level1_load_state.keyboard_layout = 'second_state'

local state = level1_load_state

--##########################################################################--
--[[----------------------------------------------------------------------]]--
--      INPUT
--[[----------------------------------------------------------------------]]--
--##########################################################################--
function level1_load_state.keypressed(key)
end
function level1_load_state.keyreleased(key)
end
function level1_load_state.mousepressed(x, y, button)
end
function level1_load_state.mousereleased(x, y, button)
end

--##########################################################################--
--[[----------------------------------------------------------------------]]--
--      LOAD
--[[----------------------------------------------------------------------]]--
--##########################################################################--
function level1_load_state.load(str)
  -- construct level
  local camera = camera2d:new()
  camera:set_target(vector2:new(20000, 35000), true)
  
  local level = level:new()
  level:set_camera(camera)
  state.level = level
  
  -- colours
  local C_BLACK = {0, 0, 0, 255}
  local ncolors = 3 * math.floor(MAX_IMAGE_WIDTH / TILE_WIDTH)
  local blend = 0.03
  local g_background = tile_gradient:new(require('gradients/test_gradient1'), 
                                         C_BLACK, blend, ncolors)
  local g_wall1 =       tile_gradient:new(require('gradients/test_gradient2'), 
                                         C_BLACK, blend, ncolors, true)
  local g_wall2 =  tile_gradient:new(require('gradients/test_gradient3'), 
                                         C_BLACK, blend, ncolors, true)
  local g_wall3 =  tile_gradient:new(require('gradients/test_gradient4'), 
                                         C_BLACK, blend, ncolors, true)

  local palette = tile_palette:new()
  palette:add_gradient(g_background, 1)
  palette:add_gradient(g_wall1, 2)
  palette:add_gradient(g_wall2, 3)
  palette:add_gradient(g_wall3, 4)
  palette:generate_spritebatch_image()
  
  -- load map
  local map_directory = "images/level4"
  local map_directories = lf.getDirectoryItems(map_directory)
  local back = "background_layer.png"
  local wall_small = "wall_small_layer.png"
  local wall_medium = "wall_medium_layer.png"
  local wall_large = "wall_large_layer.png"
  
  level_map = level_map:new(level)
  for i=1,#map_directories do
  	local dir_name = map_directories[i]
  	local dir = map_directory..'/'..dir_name
  	local offx, offy = dir_name:match("([^,]+)-([^,]+)")
  	
  	local tmap = tile_map:new(level, 100, 100)
  	
  	if lf.exists(dir..'/'..back) then
  		local layer = tile_layer:new(dir..'/'..back, palette:get_gradient(1), 0, T_WALK)
      tmap:add_tile_layer(layer)
  	end
  	if lf.exists(dir..'/'..wall_small) then
  		local layer = tile_layer:new(dir..'/'..wall_small, palette:get_gradient(2), 0, T_WALL)
      tmap:add_tile_layer(layer)
  	end
  	if lf.exists(dir..'/'..wall_medium) then
  		local layer = tile_layer:new(dir..'/'..wall_medium, palette:get_gradient(3), 0, T_WALL)
      tmap:add_tile_layer(layer)
  	end
  	if lf.exists(dir..'/'..wall_large) then
  		local layer = tile_layer:new(dir..'/'..wall_large, palette:get_gradient(4), 0, T_WALL)
      tmap:add_tile_layer(layer)
  	end
  	
  	level_map:add_tile_map(tmap, offx, offy)
  end
  level_map:load()
  
end


--##########################################################################--
--[[----------------------------------------------------------------------]]--
--      UPDATE
--[[----------------------------------------------------------------------]]--
--##########################################################################--
function level1_load_state.update(dt)
  local level = state.level
  level:update(dt)
  
  local map = level:get_level_map()
  if map:is_loaded() then
    invaders:load_next_state(level)
  end
end
  

--##########################################################################--
--[[----------------------------------------------------------------------]]--
--     DRAW
--[[----------------------------------------------------------------------]]--
--##########################################################################--
function level1_load_state.draw()
  state.level:draw()
end

return level1_load_state


















